@@ -6,8 +6,10 @@
 #define QFLOAT16_H
 
 #include <QtCore/qglobal.h>
+#include <QtCore/qhashfunctions.h>
 #include <QtCore/qmath.h>
 #include <QtCore/qnamespace.h>
+
 #include <limits>
 #include <string.h>
 
@@ -85,7 +87,7 @@ public:
     inline operator float() const noexcept;
 #endif
     template <typename T, typename = std::enable_if_t<std::is_arithmetic_v<T> && !std::is_same_v<T, NearestFloat>>>
-    explicit qfloat16(T value) noexcept : qfloat16(NearestFloat(value)) {}
+    constexpr explicit qfloat16(T value) noexcept : qfloat16(NearestFloat(value)) {}
 
     // Support for qIs{Inf,NaN,Finite}:
     bool isInf() const noexcept { return (b16 & 0x7fff) == 0x7c00; }
@@ -144,6 +146,9 @@ private:
     friend inline qfloat16 operator*(qfloat16 a, qfloat16 b) noexcept { return qfloat16(static_cast<NearestFloat>(a) * static_cast<NearestFloat>(b)); }
     friend inline qfloat16 operator/(qfloat16 a, qfloat16 b) noexcept { return qfloat16(static_cast<NearestFloat>(a) / static_cast<NearestFloat>(b)); }
 
+    friend size_t qHash(qfloat16 key, size_t seed = 0) noexcept
+    { return qHash(float(key), seed); } // 6.4 algorithm, so keep using it; ### Qt 7: fix QTBUG-116077
+
 #define QF16_MAKE_ARITH_OP_FP(FP, OP) \
     friend inline FP operator OP(qfloat16 lhs, FP rhs) noexcept { return static_cast<FP>(lhs) OP rhs; } \
     friend inline FP operator OP(FP lhs, qfloat16 rhs) noexcept { return lhs OP static_cast<FP>(rhs); }
