@@ -104,7 +104,7 @@
 
 #define QT_FEATURE_xcb_sm 1
 
-#define QT_FEATURE_system_xcb_xinput -1
+#define QT_FEATURE_system_xcb_xinput 1
 
 #define QT_FEATURE_xkbcommon 1
 
