@@ -5,13 +5,13 @@
 #define QMESSAGEBOX_H
 
 #include <QtWidgets/qtwidgetsglobal.h>
-
 #include <QtWidgets/qdialog.h>
 
 QT_REQUIRE_CONFIG(messagebox);
 
 QT_BEGIN_NAMESPACE
 
+class QAnyStringView;
 class QLabel;
 class QMessageBoxPrivate;
 class QAbstractButton;
@@ -31,10 +31,16 @@ class Q_WIDGETS_EXPORT QMessageBox : public QDialog
     Q_PROPERTY(QString informativeText READ informativeText WRITE setInformativeText)
     Q_PROPERTY(Qt::TextInteractionFlags textInteractionFlags READ textInteractionFlags
                WRITE setTextInteractionFlags)
-
+    Q_PROPERTY(Options options READ options WRITE setOptions)
 public:
+    // Keep in sync with MessageBoxOption in qplatformdialoghelper.h
+    enum class Option {
+        DontUseNativeDialog = 0x00000001
+    };
+    Q_FLAG(Option)
+
     enum Icon {
-        // keep this in sync with QMessageDialogOptions::Icon
+        // keep this in sync with QMessageDialogOptions::StandardIcon
         NoIcon = 0,
         Information = 1,
         Warning = 2,
@@ -95,8 +101,9 @@ public:
 #if QT_VERSION < QT_VERSION_CHECK(7, 0, 0)
     typedef StandardButton Button;
 #endif
-
+    Q_DECLARE_FLAGS(Options, Option)
     Q_DECLARE_FLAGS(StandardButtons, StandardButton)
+
     Q_FLAG(StandardButtons)
 
     explicit QMessageBox(QWidget *parent = nullptr);
@@ -149,6 +156,11 @@ public:
     void setCheckBox(QCheckBox *cb);
     QCheckBox* checkBox() const;
 
+    void setOption(Option option, bool on = true);
+    bool testOption(Option option) const;
+    void setOptions(Options options);
+    Options options() const;
+
     static StandardButton information(QWidget *parent, const QString &title,
          const QString &text, StandardButtons buttons = Ok,
          StandardButton defaultButton = NoButton);
@@ -292,7 +304,7 @@ protected:
 
 private:
     Q_PRIVATE_SLOT(d_func(), void _q_buttonClicked(QAbstractButton *))
-    Q_PRIVATE_SLOT(d_func(), void _q_clicked(QPlatformDialogHelper::StandardButton, QPlatformDialogHelper::ButtonRole))
+    Q_PRIVATE_SLOT(d_func(), void _q_helperClicked(QPlatformDialogHelper::StandardButton, QPlatformDialogHelper::ButtonRole))
 
     Q_DISABLE_COPY(QMessageBox)
     Q_DECLARE_PRIVATE(QMessageBox)
@@ -300,20 +312,9 @@ private:
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QMessageBox::StandardButtons)
 
-#define QT_REQUIRE_VERSION(argc, argv, str) { QString s = QString::fromLatin1(str);\
-QString sq = QString::fromLatin1(qVersion()); \
-if ((sq.section(QChar::fromLatin1('.'),0,0).toInt()<<16)+\
-(sq.section(QChar::fromLatin1('.'),1,1).toInt()<<8)+\
-sq.section(QChar::fromLatin1('.'),2,2).toInt()<(s.section(QChar::fromLatin1('.'),0,0).toInt()<<16)+\
-(s.section(QChar::fromLatin1('.'),1,1).toInt()<<8)+\
-s.section(QChar::fromLatin1('.'),2,2).toInt()) { \
-if (!qApp){ \
-    new QApplication(argc,argv); \
-} \
-QString s = QApplication::tr("Executable '%1' requires Qt "\
- "%2, found Qt %3.").arg(qAppName()).arg(QString::fromLatin1(\
-str)).arg(QString::fromLatin1(qVersion())); QMessageBox::critical(0, QApplication::tr(\
-"Incompatible Qt Library Error"), s, QMessageBox::Abort, 0); qFatal("%s", s.toLatin1().data()); }}
+Q_WIDGETS_EXPORT void qRequireVersion(int argc, char *argv[], QAnyStringView req);
+
+#define QT_REQUIRE_VERSION(argc, argv, str) qRequireVersion(argc, argv, str);
 
 QT_END_NAMESPACE
 
